/* ================================================================
`\itshape 三维几何`
*/


struct			Tpoint
{
	double		x , y , z;
	Tpoint(){}
	Tpoint(double x0, double y0, double z0) : x(x0), y(y0), z(z0) {}
};

//`\itshape 计算两个向量的叉积`
Tpoint	crossProduct(const Tpoint& A, const Tpoint& B)
{
	return	Tpoint(A.y*B.z - A.z*B.y, A.z*B.x - A.x*B.z, A.x*B.y - A.y*B.x);
}

//`\itshape 计算两个向量的点积`
double	dotProduct(const Tpoint& A , const Tpoint& B)
{
	return	A.x * B.x + A.y * B.y + A.z * B.z;
}

struct			Tplane
{
	Tpoint		p;	// `\itshape 平面上一个点`
	Tpoint		n;	// `\itshape 法向量`

	Tplane(){}
	Tplane(const Tpoint& p0, const Tpoint& n0) : p(p0), n(n0) {}
	Tplane(const Tpoint& A , const Tpoint& B , const Tpoint& C)
	{
		p = A;
		n = crossProduct(B - A , C - A);
	}
};

//`\itshape 计算向量A 在P 方向上的投影`
Tpoint	project(const Tpoint& A , const Tpoint& P)
{
	double	k = dotProduct(A,P) / P.norm();
	return	Tpoint(P.x * k , P.y * k , P.z * k);
}

//`\itshape 判断直线AB 和PQ是否平行`
bool	isParallel(const Tpoint& A, const Tpoint& B, const Tpoint&P, const Tpoint& Q)
{
	Tpoint	S = crossProduct(B - A , Q - P);
	return	sgn( S.x ) == 0 && sgn( S.y ) == 0 && sgn( S.z ) == 0;
}

//`\itshape 判断直线AB 和PQ是否垂直`
bool	isChuiZhi(const Tpoint& A, const Tpoint& B, const Tpoint& P, const Tpoint& Q)
{
	return	sgn( dotProduct(B - A, Q - P) ) == 0;
}

//`\itshape 判断点是否在直线上`
bool	onSegment(const Tpoint& A , const Tpoint& B , const Tpoint& P)
{
	return	isParallel(A , B , A , P) &&
		sgn( dotProduct(A - P , B - P) ) <= 0;
}

//`\itshape 判两点P,Q在直线AB的同侧(+1) 异侧(-1) 一个点在线上(0), 不共面无意义`
int	checkSide(const Tpoint& A , const Tpoint& B , const Tpoint& P , const Tpoint& Q)
{
	return	sgn( dotProduct( crossProduct(B-A,P-A) , crossProduct(B-A,Q-A) ) );
}

//`\itshape 求点P到直线AB的距离`
double	distToLine(const Tpoint& A , const Tpoint& B , const Tpoint& P)
{
	return	crossProduct(B - P , A - P).norm() / (A - B).norm();
}

//`\itshape 求点P到直线AB上的垂足`
Tpoint	getH(const Tpoint& A , const Tpoint& B , const Tpoint& P)
{
	double	t = dotProduct( P - A , B - A ) / dotProduct( B - A , B - A );
	return	A + (B - A) * t;
}

//`\itshape 判断是否四点共面`
bool	dotsOnPlane(const Tpoint& A , const Tpoint& B , const Tpoint& C , const Tpoint& D)
{
	return	sgn( dotProduct( crossProduct(B - A , C - A) , D - A ) ) == 0;
}

//`\itshape 计算点A 到平面S的距离		: PA到S.n上的投影的长度`
double	distToPlane(const Tpoint& A , const Tplane& S)
{
	return	fabs( dotProduct(A - S.p , S.n) / S.n.norm() );
}

//`\itshape 求直线AB 与平面S的交点`
//`\itshape 需要保证AB与S不平行, 即dotProduct( (AB), S.n ) != 0`
Tpoint	crossPointOnPlane(const Tpoint& A , const Tpoint& B , const Tplane& S)
{
	double		t = dotProduct( S.n , S.p - A ) / dotProduct(S.n , B - A);
	return	A + (B - A) * t;
}

//`\itshape 判断点P是否在空间三角形ABC内`
//`\itshape 判面积( 叉积的模/ 2 ) , 判点积( 判边界)`





//`\itshape 判段两共面线段是否严格相交`
bool	segmentCross(const Tpoint& A , const Tpoint& B , const Tpoint& C , const Tpoint& D)
{
	return	dotsOnPlane(A,B,C,D) && checkSide(A,B,C,D) < 0 && checkSide(C,D,A,B) < 0;
}

//`\itshape 计算两相交直线交点(共面且不平行)`
Tpoint	crossPoint(const Tpoint& A , const Tpoint& B , const Tpoint& P , const Tpoint& Q)
{
	double	t;
	if ( sgn( (B.x - A.x) * (Q.y - P.y) - (B.y - A.y) * (Q.x - P.x) ) )
		t = ( (P.x - A.x) * (Q.y - P.y) - (P.y - A.y) * (Q.x - P.x) )
		/   ( (B.x - A.x) * (Q.y - P.y) - (B.y - A.y) * (Q.x - P.x) );

	else if ( sgn( (B.y - A.y) * (Q.z - P.z) - (B.z - A.z) * (Q.y - P.y) ) )
		t = ( (P.y - A.y) * (Q.z - P.z) - (P.z - A.z) * (Q.y - P.y) )
		/   ( (B.y - A.y) * (Q.z - P.z) - (B.z - A.z) * (Q.y - P.y) );

	else	t = ( (P.z - A.z) * (Q.x - P.x) - (P.x - A.x) * (Q.z - P.z) )
		/   ( (B.z - A.z) * (Q.x - P.x) - (B.x - A.x) * (Q.z - P.z) );

	return	A + (B - A) * t;
}

//`\itshape 计算两条直线AB, CD的距离`
double	lineDist(const Tpoint& A , const Tpoint& B , const Tpoint& P , const Tpoint& Q)
{
	Tpoint	n = crossProduct(B - A , Q - P);
	if (n.norm() < eps) //`\itshape 平行`
		return	distToLine(A , B , P);
	return	fabs( dotProduct(P - A , n) / n.norm() );
}

//`\itshape 计算直线AB, CD的最近点, 返回(X,Y) X in AB, Y in PQ`
void	nearestPoints(const Tpoint& A, const Tpoint& B, const Tpoint& P , const Tpoint& Q , Tpoint& X , Tpoint& Y)
{
	if (isParallel(A,B,P,Q)) return;
	
	double	a1 = + dotProduct( B - A , B - A );
	double	b1 = - dotProduct( B - A , Q - P );
	double	c1 = - dotProduct( B - A , A - P );

	double	a2 = + dotProduct( Q - P , B - A );
	double	b2 = - dotProduct( Q - P , Q - P );
	double	c2 = - dotProduct( Q - P , A - P );

	double	t1 = (c1 * b2 - c2 * b1) / (a1 * b2 - a2 * b1);
	double	t2 = (c2 - t1 * a2) / b2;

	X = A + (B - A) * t1;
	Y = P + (Q - P) * t2;
}
